==> flutter_webview_desktop-0.0.3-dev.tar.gz: windows/webview_bridge.cc <==
#include "webview_bridge.h"
#include "include/views/cef_browser_view.h"
#include "include/views/cef_window.h"
#include "include/wrapper/cef_helpers.h"
#include <flutter/texture_registrar.h>
#include <flutter/event_stream_handler_functions.h>
#include <flutter/method_result_functions.h>
#include <fmt/core.h>
#include <memory>
#include <optional>

constexpr auto kErrorInvalidArgs = "invalidArguments";

constexpr auto kMethodLoadUrl = "loadUrl";
constexpr auto kMethodLoaddHtmlString = "loadHtmlString";
constexpr auto kMethodLoadRequest = "loadRequest";

constexpr auto kMethodReload = "reload";
constexpr auto kMethodStop = "stop";
constexpr auto kMethodGoBack = "goBack";
constexpr auto kMethodGoForward = "goForward";

constexpr auto kMethodSetViewSize = "setViewSize";
// constexpr auto kMethodSetCursorPos = "setCursorOffset";
constexpr auto kMethodScrolllDelta = "scrollDelta";

constexpr auto kMethodClearCache = "clearCache";

constexpr auto kMethodSetMouseClick = "setMouseClick";

constexpr auto kMethodSetWindowOpenPolicy = "setWindowOpenPolicy";
constexpr auto kMethodSetUserAgent = "setUserAgent";

constexpr auto kWebviewEventType = "type";
constexpr auto kWebviewEventValue = "value";

constexpr auto kMethodEvaluateJavascript = "evaluateJavascript";

namespace
{
    static const std::optional<std::pair<double, double>> GetPointFromArgs(
        const flutter::EncodableValue *args)
    {
        const flutter::EncodableList *list =
            std::get_if<flutter::EncodableList>(args);
        if (!list || list->size() != 2)
        {
            return std::nullopt;
        }
        const auto x = std::get_if<double>(&(*list)[0]);
        const auto y = std::get_if<double>(&(*list)[1]);
        if (!x || !y)
        {
            return std::nullopt;
        }
        return std::make_pair(*x, *y);
    }
    static const std::string &GetWinCursorName(const HCURSOR cursor)
    {

        static const std::string kDefaultCursorName = "basic";
        static const std::pair<std::string, const wchar_t *> mappings[] = {
            {"allScroll", IDC_SIZEALL},
            {kDefaultCursorName, IDC_ARROW},
            {"click", IDC_HAND},
            {"forbidden", IDC_NO},
            {"help", IDC_HELP},
            {"move", IDC_SIZEALL},
            {"none", nullptr},
            {"noDrop", IDC_NO},
            {"precise", IDC_CROSS},
            {"progress", IDC_APPSTARTING},
            {"text", IDC_IBEAM},
            {"resizeColumn", IDC_SIZEWE},
            {"resizeDown", IDC_SIZENS},
            {"resizeDownLeft", IDC_SIZENESW},
            {"resizeDownRight", IDC_SIZENWSE},
            {"resizeLeft", IDC_SIZEWE},
            {"resizeLeftRight", IDC_SIZEWE},
            {"resizeRight", IDC_SIZEWE},
            {"resizeRow", IDC_SIZENS},
            {"resizeUp", IDC_SIZENS},
            {"resizeUpDown", IDC_SIZENS},
            {"resizeUpLeft", IDC_SIZENWSE},
            {"resizeUpRight", IDC_SIZENESW},
            {"resizeUpLeftDownRight", IDC_SIZENWSE},
            {"resizeUpRightDownLeft", IDC_SIZENESW},
            {"wait", IDC_WAIT},
        };

        static std::map<HCURSOR, std::string> cursors;
        static bool initialized = false;

        if (!initialized)
        {
            initialized = true;
            for (const auto &pair : mappings)
            {
                HCURSOR cursor_handle = LoadCursor(nullptr, pair.second);
                if (cursor_handle)
                {
                    cursors[cursor_handle] = pair.first;
                }
            }
        }

        const auto it = cursors.find(cursor);
        if (it != cursors.end())
        {
            return it->second;
        }
        return kDefaultCursorName;
    }
}

// namespace
WebviewBridge::WebviewBridge(flutter::TextureRegistrar *texture_registrar, flutter::BinaryMessenger *messenger, CefRefPtr<WebviewClient> webview) : texture_registrar_(texture_registrar), binary_messenger_(messenger), webview_client_(webview)
{
    flutter_texture_ = std::make_unique<flutter::TextureVariant>(flutter::PixelBufferTexture(
        [this](size_t width, size_t height)
            -> const FlutterDesktopPixelBuffer *
        {
            return webview_client_->CopyPixelBuffer(width, height);
        }));
    texture_id_ = texture_registrar_->RegisterTexture(flutter_texture_.get());
    texture_registrar_->MarkTextureFrameAvailable(texture_id_);

    webview_client_->SetOnFrameAvailable(
        [this]()
        {
            texture_registrar_->MarkTextureFrameAvailable(texture_id_);
        });

    const auto method_channel_name =
        fmt::format("flutter_webview_desktop/{}", texture_id_);
    method_channel_ =
        std::make_unique<flutter::MethodChannel<flutter::EncodableValue>>(
            binary_messenger_, method_channel_name,
            &flutter::StandardMethodCodec::GetInstance());
    method_channel_->SetMethodCallHandler([this](const auto &call, auto result)
                                          { HandleMethodCall(call, std::move(result)); });

    const auto event_channel_name =
        fmt::format("flutter_webview_desktop/{}/events", texture_id_);
    event_channel_ =
        std::make_unique<flutter::EventChannel<flutter::EncodableValue>>(
            messenger, event_channel_name,
            &flutter::StandardMethodCodec::GetInstance());

    auto handler = std::make_unique<
        flutter::StreamHandlerFunctions<flutter::EncodableValue>>(
        [this](const flutter::EncodableValue *arguments,
               std::unique_ptr<flutter::EventSink<flutter::EncodableValue>> &&
                   events)
        {
            event_sink_ = std::move(events);
            RegisterEventHandlers();
            return nullptr;
        },
        [](const flutter::EncodableValue *arguments)
        { return nullptr; });

    event_channel_->SetStreamHandler(std::move(handler));
}
WebviewBridge::~WebviewBridge()
{
    texture_registrar_->UnregisterTexture(texture_id_);
}
/// dart -> natvie
void WebviewBridge::HandleMethodCall(
    const flutter::MethodCall<flutter::EncodableValue> &method_call,
    std::unique_ptr<flutter::MethodResult<flutter::EncodableValue>> result)
{
    const auto &method_name = method_call.method_name();

    // loadUrl: string
    if (method_name.compare(kMethodLoadUrl) == 0)
    {
        // todo 修改
        if (const auto url = std::get_if<std::string>(method_call.arguments()))
        {
            webview_client_->LoadUrl(*url);
            return result->Success();
        }
        return result->Error(kErrorInvalidArgs);
    }

    // loadStringContent: string
    if (method_name.compare(kMethodLoaddHtmlString) == 0)
    {
        // todo 修改
        if (const auto content =
                std::get_if<std::string>(method_call.arguments()))
        {
            webview_client_->LoadStringContent(*content);
            return result->Success();
        }
        return result->Error(kErrorInvalidArgs);
    }

    if (method_name.compare(kMethodLoadRequest))
    {
        // todo
        // return result->NotImplemented();
    }

    // reload
    if (method_name.compare(kMethodReload) == 0)
    {
        webview_client_->Reload();
        return result->Success();
    }

    // stop
    if (method_name.compare(kMethodStop) == 0)
    {
        webview_client_->StopLoad();
        return result->Success();
    }

    // goBack
    if (method_name.compare(kMethodGoBack) == 0)
    {
        webview_client_->GoBack();
        return result->Success();
    }

    // goForward
    if (method_name.compare(kMethodGoForward) == 0)
    {
        webview_client_->GoForward();
        return result->Success();
    }

    // setSize: [double width, double height]
    if (method_name.compare(kMethodSetViewSize) == 0)
    {
        auto size = GetPointFromArgs(method_call.arguments());
        if (size)
        {
            webview_client_->SetTextureSize(static_cast<int>(size->first),
                                            static_cast<int>(size->second));

            // texture_bridge_->Start();
            return result->Success();
        }
        return result->Error(kErrorInvalidArgs);
    }

    /// scrollBy
    if (method_name.compare(kMethodScrolllDelta) == 0)
    {
        const auto &map = std::get<flutter::EncodableMap>(*method_call.arguments());
        const auto x = map.find(flutter::EncodableValue("x"));
        const auto y = map.find(flutter::EncodableValue("y"));
        const auto deltaX = map.find(flutter::EncodableValue("deltaX"));
        const auto deltaY = map.find(flutter::EncodableValue("deltaY"));
        const auto xValue = std::get_if<int>(&x->second);
        const auto yValue = std::get_if<int>(&y->second);
        const auto deltaXValue = std::get_if<int>(&deltaX->second);
        const auto deltaYValue = std::get_if<int>(&deltaY->second);
        webview_client_->SetMouseWheel(*xValue, *yValue, *deltaXValue, *deltaYValue);
        return result->Success();
    }

    // /// clearCache
    //  if (method_name.compare(kMethodClearCache))
    // {
    //     // return result->NotImplemented();
    // }
    if (method_call.method_name().compare(kMethodEvaluateJavascript) == 0)
    {
        const auto &map = std::get<flutter::EncodableMap>(*method_call.arguments());
        const auto url = map.find(flutter::EncodableValue("url"));
        const auto code = map.find(flutter::EncodableValue("code"));

        const auto urlValue = std::get_if<std::string>(&url->second);
        const auto codeValue = std::get_if<std::string>(&code->second);
        webview_client_->EvaluateJavaScript(*urlValue, *codeValue);

        // const auto event = flutter::EncodableValue("success");
        return result->Success();
    }
    // setMouseClick
    if (method_name.compare(kMethodSetMouseClick) == 0)
    {
        const auto &map = std::get<flutter::EncodableMap>(*method_call.arguments());

        const auto button = map.find(flutter::EncodableValue("button"));
        const auto isDown = map.find(flutter::EncodableValue("isDown"));
        const auto x = map.find(flutter::EncodableValue("x"));
        const auto y = map.find(flutter::EncodableValue("y"));
        ;
        if (button != map.end() && isDown != map.end())
        {
            const auto buttonValue = std::get_if<int32_t>(&button->second);
            const auto xValue = std::get_if<int>(&x->second);
            const auto yValue = std::get_if<int>(&y->second);
            const auto isDownValue = std::get_if<bool>(&isDown->second);
            if (buttonValue && isDownValue)
            {
                webview_client_->SetMouseClick(
                    static_cast<WebviewClientPointerButton>(*buttonValue), *isDownValue, *xValue, *yValue);
                return result->Success();
            }
        }
        return result->Error(kErrorInvalidArgs);
    }

    // setWindowOpenPolicy
    if (method_name.compare(kMethodSetWindowOpenPolicy) == 0)
    {
        if (const auto index = std::get_if<int32_t>(method_call.arguments()))
        {
            switch (*index)
            {
            case 1:
                webview_client_->SetWindowOpenPolicy(WebviewClientWindowOpenPolicy::Deny);
                break;
            case 2:
                webview_client_->SetWindowOpenPolicy(
                    WebviewClientWindowOpenPolicy::InSameWindow);
                break;
            default:
                webview_client_->SetWindowOpenPolicy(WebviewClientWindowOpenPolicy::Allow);
                break;
            }
            return result->Success();
        }
        return result->Error(kErrorInvalidArgs);
    }
}

/// native -> dart
void WebviewBridge::RegisterEventHandlers()
{

    webview_client_->OnCursorChanged([this](const HCURSOR cursor)
                                     {
                                         const auto &name = GetWinCursorName(cursor);
                                         const auto event = flutter::EncodableValue(
                                             flutter::EncodableMap{{flutter::EncodableValue(kWebviewEventType),
                                                                    flutter::EncodableValue("onCursorChanged")},
                                                                   {flutter::EncodableValue(kWebviewEventValue), name}});
                                         event_sink_->Success(event); });

    webview_client_->OnLoadStateChanged([this](WebviewClientLoadState loadState)
                                        {
                                            const auto event = flutter::EncodableValue(
                                                flutter::EncodableMap{{flutter::EncodableValue(kWebviewEventType),
                                                                       flutter::EncodableValue("loadStateChanged")},
                                                                      {flutter::EncodableValue(kWebviewEventValue), static_cast<int>(loadState)}});

                                            event_sink_->Success(event); });

    webview_client_->OnHistoryChanged([this](WebviewClientHistoryChanged historyChanged)
                                      {
                                          const auto event = flutter::EncodableValue(
                                              flutter::EncodableMap{
                                                  {flutter::EncodableValue(kWebviewEventType),
                                                   flutter::EncodableValue("onHistoryChanged")},
                                                  {flutter::EncodableValue(kWebviewEventValue),
                                                   flutter::EncodableValue(flutter::EncodableMap{
                                                       {flutter::EncodableValue("canGoBack"),
                                                        flutter::EncodableValue(
                                                            static_cast<bool>(historyChanged.can_go_back))},
                                                       {flutter::EncodableValue("canGoForward"),
                                                        flutter::EncodableValue(
                                                            static_cast<bool>(historyChanged.can_go_forward))},
                                                   })},});

                                          event_sink_->Success(event); });

    webview_client_->OnUrlChanged([this](const std::string &url)
                                  {
                                      const auto event = flutter::EncodableValue(
                                          flutter::EncodableMap{{flutter::EncodableValue(kWebviewEventType),
                                                                 flutter::EncodableValue("urlChanged")},
                                                                {flutter::EncodableValue(kWebviewEventValue), static_cast<std::string>(url)}});

                                      event_sink_->Success(event); });

    webview_client_->OnJSHandleChanged([this](const std::string &name )
                                       {
      // TODO 
        ::MessageBox(NULL, L"OnJSHandleChanged", L"Tip", MB_OK);
        // const auto event = flutter::EncodableValue(
        //     flutter::EncodableMap{
        //         {flutter::EncodableValue(kWebviewEventType),
        //          flutter::EncodableValue("javascriptChannelMessage")},
        //         {
        //             flutter::EncodableValue(kWebviewEventValue),
        //             flutter::EncodableValue(flutter::EncodableMap{
        //                 {flutter::EncodableValue("name"),
        //                  flutter::EncodableValue(
        //                      name)},
        //                 {flutter::EncodableValue("result"),
        //                  flutter::EncodableValue(
        //                      result)},
        //             }),
        //         },
        //     });
        // event_sink_->Success(event);
         });
}

==> flutter_webview_desktop-0.0.8-dev.tar.gz: windows/webview_bridge.cc <==
#include "webview_bridge.h"
#include "include/views/cef_browser_view.h"
#include "include/views/cef_window.h"
#include "include/wrapper/cef_helpers.h"
#include <flutter/texture_registrar.h>
#include <flutter/event_stream_handler_functions.h>
#include <flutter/method_result_functions.h>
#include <fmt/core.h>
#include <memory>
#include <optional>

constexpr auto kErrorInvalidArgs = "invalidArguments";

constexpr auto kMethodLoadUrl = "loadUrl";
constexpr auto kMethodLoaddHtmlString = "loadHtmlString";
constexpr auto kMethodLoadRequest = "loadRequest";

constexpr auto kMethodReload = "reload";
constexpr auto kMethodStop = "stop";
constexpr auto kMethodGoBack = "goBack";
constexpr auto kMethodGoForward = "goForward";

constexpr auto kMethodSetViewSize = "setViewSize";
// constexpr auto kMethodSetCursorPos = "setCursorOffset";
constexpr auto kMethodScrolllDelta = "scrollDelta";

constexpr auto kMethodClearCache = "clearCache";

constexpr auto kMethodSetMouseClick = "setMouseClick";

constexpr auto kMethodSetWindowOpenPolicy = "setWindowOpenPolicy";
constexpr auto kMethodSetUserAgent = "setUserAgent";

constexpr auto kWebviewEventType = "type";
constexpr auto kWebviewEventValue = "value";

constexpr auto kMethodEvaluateJavascript = "evaluateJavascript";

namespace
{
    static const std::optional<std::pair<double, double>> GetPointFromArgs(
        const flutter::EncodableValue *args)
    {
        const flutter::EncodableList *list =
            std::get_if<flutter::EncodableList>(args);
        if (!list || list->size() != 2)
        {
            return std::nullopt;
        }
        const auto x = std::get_if<double>(&(*list)[0]);
        const auto y = std::get_if<double>(&(*list)[1]);
        if (!x || !y)
        {
            return std::nullopt;
        }
        return std::make_pair(*x, *y);
    }
    static const std::string &GetWinCursorName(const HCURSOR cursor)
    {

        static const std::string kDefaultCursorName = "basic";
        static const std::pair<std::string, const wchar_t *> mappings[] = {
            {"allScroll", IDC_SIZEALL},
            {kDefaultCursorName, IDC_ARROW},
            {"click", IDC_HAND},
            {"forbidden", IDC_NO},
            {"help", IDC_HELP},
            {"move", IDC_SIZEALL},
            {"none", nullptr},
            {"noDrop", IDC_NO},
            {"precise", IDC_CROSS},
            {"progress", IDC_APPSTARTING},
            {"text", IDC_IBEAM},
            {"resizeColumn", IDC_SIZEWE},
            {"resizeDown", IDC_SIZENS},
            {"resizeDownLeft", IDC_SIZENESW},
            {"resizeDownRight", IDC_SIZENWSE},
            {"resizeLeft", IDC_SIZEWE},
            {"resizeLeftRight", IDC_SIZEWE},
            {"resizeRight", IDC_SIZEWE},
            {"resizeRow", IDC_SIZENS},
            {"resizeUp", IDC_SIZENS},
            {"resizeUpDown", IDC_SIZENS},
            {"resizeUpLeft", IDC_SIZENWSE},
            {"resizeUpRight", IDC_SIZENESW},
            {"resizeUpLeftDownRight", IDC_SIZENWSE},
            {"resizeUpRightDownLeft", IDC_SIZENESW},
            {"wait", IDC_WAIT},
        };

        static std::map<HCURSOR, std::string> cursors;
        static bool initialized = false;

        if (!initialized)
        {
            initialized = true;
            for (const auto &pair : mappings)
            {
                HCURSOR cursor_handle = LoadCursor(nullptr, pair.second);
                if (cursor_handle)
                {
                    cursors[cursor_handle] = pair.first;
                }
            }
        }

        const auto it = cursors.find(cursor);
        if (it != cursors.end())
        {
            return it->second;
        }
        return kDefaultCursorName;
    }
}

// namespace
WebviewBridge::WebviewBridge(flutter::TextureRegistrar *texture_registrar, flutter::BinaryMessenger *messenger, CefRefPtr<WebviewClient> webview) : texture_registrar_(texture_registrar), binary_messenger_(messenger), webview_client_(webview)
{
    flutter_texture_ = std::make_unique<flutter::TextureVariant>(flutter::PixelBufferTexture(
        [this](size_t width, size_t height)
            -> const FlutterDesktopPixelBuffer *
        {
            return webview_client_->CopyPixelBuffer(width, height);
        }));
    texture_id_ = texture_registrar_->RegisterTexture(flutter_texture_.get());
    texture_registrar_->MarkTextureFrameAvailable(texture_id_);

    webview_client_->SetOnFrameAvailable(
        [this]()
        {
            texture_registrar_->MarkTextureFrameAvailable(texture_id_);
        });

    const auto method_channel_name =
        fmt::format("flutter_webview_desktop/{}", texture_id_);
    method_channel_ =
        std::make_unique<flutter::MethodChannel<flutter::EncodableValue>>(
            binary_messenger_, method_channel_name,
            &flutter::StandardMethodCodec::GetInstance());
    method_channel_->SetMethodCallHandler([this](const auto &call, auto result)
                                          { HandleMethodCall(call, std::move(result)); });

    const auto event_channel_name =
        fmt::format("flutter_webview_desktop/{}/events", texture_id_);
    event_channel_ =
        std::make_unique<flutter::EventChannel<flutter::EncodableValue>>(
            messenger, event_channel_name,
            &flutter::StandardMethodCodec::GetInstance());

    auto handler = std::make_unique<
        flutter::StreamHandlerFunctions<flutter::EncodableValue>>(
        [this](const flutter::EncodableValue *arguments,
               std::unique_ptr<flutter::EventSink<flutter::EncodableValue>> &&
                   events)
        {
            event_sink_ = std::move(events);
            RegisterEventHandlers();
            return nullptr;
        },
        [](const flutter::EncodableValue *arguments)
        { return nullptr; });

    event_channel_->SetStreamHandler(std::move(handler));
}
WebviewBridge::~WebviewBridge()
{
    texture_registrar_->UnregisterTexture(texture_id_);
}
/// dart -> natvie
void WebviewBridge::HandleMethodCall(
    const flutter::MethodCall<flutter::EncodableValue> &method_call,
    std::unique_ptr<flutter::MethodResult<flutter::EncodableValue>> result)
{
    const auto &method_name = method_call.method_name();

    // loadUrl: string
    if (method_name.compare(kMethodLoadUrl) == 0)
    {
        // todo 修改
        if (const auto url = std::get_if<std::string>(method_call.arguments()))
        {
            webview_client_->LoadUrl(*url);
            return result->Success();
        }
        return result->Error(kErrorInvalidArgs);
    }

    // loadStringContent: string
    if (method_name.compare(kMethodLoaddHtmlString) == 0)
    {
        // todo 修改
        if (const auto content =
                std::get_if<std::string>(method_call.arguments()))
        {
            webview_client_->LoadStringContent(*content);
            return result->Success();
        }
        return result->Error(kErrorInvalidArgs);
    }

    if (method_name.compare(kMethodLoadRequest))
    {
        // todo
        // return result->NotImplemented();
    }

    // reload
    if (method_name.compare(kMethodReload) == 0)
    {
        webview_client_->Reload();
        return result->Success();
    }

    // stop
    if (method_name.compare(kMethodStop) == 0)
    {
        webview_client_->StopLoad();
        return result->Success();
    }

    // goBack
    if (method_name.compare(kMethodGoBack) == 0)
    {
        webview_client_->GoBack();
        return result->Success();
    }

    // goForward
    if (method_name.compare(kMethodGoForward) == 0)
    {
        webview_client_->GoForward();
        return result->Success();
    }

    // setSize: [double width, double height]
    if (method_name.compare(kMethodSetViewSize) == 0)
    {
        // auto size = GetPointFromArgs(method_call.arguments());
        const auto &map = std::get<flutter::EncodableMap>(*method_call.arguments());
        const auto width = map.find(flutter::EncodableValue("width"));
        const auto height = map.find(flutter::EncodableValue("height"));
        const auto devicePixelRatio = map.find(flutter::EncodableValue("devicePixelRatio"));

        const auto widthValue = std::get_if<int>(&width->second);
        const auto heightValue = std::get_if<int>(&height->second);
        const auto devicePixelRatioValue = std::get_if<double>(&devicePixelRatio->second);
        // if (size)
        // {
            webview_client_->SetTextureSize(*widthValue,
                                            *heightValue, *devicePixelRatioValue);
            return result->Success();
        // }
        // return result->Error(kErrorInvalidArgs);
    }

    /// scrollBy
    if (method_name.compare(kMethodScrolllDelta) == 0)
    {
        const auto &map = std::get<flutter::EncodableMap>(*method_call.arguments());
        const auto x = map.find(flutter::EncodableValue("x"));
        const auto y = map.find(flutter::EncodableValue("y"));
        const auto deltaX = map.find(flutter::EncodableValue("deltaX"));
        const auto deltaY = map.find(flutter::EncodableValue("deltaY"));
        const auto xValue = std::get_if<int>(&x->second);
        const auto yValue = std::get_if<int>(&y->second);
        const auto deltaXValue = std::get_if<int>(&deltaX->second);
        const auto deltaYValue = std::get_if<int>(&deltaY->second);
        webview_client_->SetMouseWheel(*xValue, *yValue, *deltaXValue, *deltaYValue);
        return result->Success();
    }

    // /// clearCache
    //  if (method_name.compare(kMethodClearCache))
    // {
    //     // return result->NotImplemented();
    // }
    if (method_call.method_name().compare(kMethodEvaluateJavascript) == 0)
    {
        const auto &map = std::get<flutter::EncodableMap>(*method_call.arguments());
        const auto url = map.find(flutter::EncodableValue("url"));
        const auto code = map.find(flutter::EncodableValue("code"));

        const auto urlValue = std::get_if<std::string>(&url->second);
        const auto codeValue = std::get_if<std::string>(&code->second);
        webview_client_->EvaluateJavaScript(*urlValue, *codeValue);

        // const auto event = flutter::EncodableValue("success");
        return result->Success();
    }
    // setMouseClick
    if (method_name.compare(kMethodSetMouseClick) == 0)
    {
        const auto &map = std::get<flutter::EncodableMap>(*method_call.arguments());

        const auto button = map.find(flutter::EncodableValue("button"));
        const auto isDown = map.find(flutter::EncodableValue("isDown"));
        const auto x = map.find(flutter::EncodableValue("x"));
        const auto y = map.find(flutter::EncodableValue("y"));
        ;
        if (button != map.end() && isDown != map.end())
        {
            const auto buttonValue = std::get_if<int32_t>(&button->second);
            const auto xValue = std::get_if<int>(&x->second);
            const auto yValue = std::get_if<int>(&y->second);
            const auto isDownValue = std::get_if<bool>(&isDown->second);
            if (buttonValue && isDownValue)
            {
                webview_client_->SetMouseClick(
                    static_cast<WebviewClientPointerButton>(*buttonValue), *isDownValue, *xValue, *yValue);
                return result->Success();
            }
        }
        return result->Error(kErrorInvalidArgs);
    }

    // setWindowOpenPolicy
    if (method_name.compare(kMethodSetWindowOpenPolicy) == 0)
    {
        if (const auto index = std::get_if<int32_t>(method_call.arguments()))
        {
            switch (*index)
            {
            case 1:
                webview_client_->SetWindowOpenPolicy(WebviewClientWindowOpenPolicy::Deny);
                break;
            case 2:
                webview_client_->SetWindowOpenPolicy(
                    WebviewClientWindowOpenPolicy::InSameWindow);
                break;
            default:
                webview_client_->SetWindowOpenPolicy(WebviewClientWindowOpenPolicy::Allow);
                break;
            }
            return result->Success();
        }
        return result->Error(kErrorInvalidArgs);
    }
}

/// native -> dart
void WebviewBridge::RegisterEventHandlers()
{

    webview_client_->OnCursorChanged([this](const HCURSOR cursor)
                                     {
                                         const auto &name = GetWinCursorName(cursor);
                                         const auto event = flutter::EncodableValue(
                                             flutter::EncodableMap{{flutter::EncodableValue(kWebviewEventType),
                                                                    flutter::EncodableValue("onCursorChanged")},
                                                                   {flutter::EncodableValue(kWebviewEventValue), name}});
                                         event_sink_->Success(event); });

    webview_client_->OnLoadStateChanged([this](WebviewClientLoadState loadState)
                                        {
                                            const auto event = flutter::EncodableValue(
                                                flutter::EncodableMap{{flutter::EncodableValue(kWebviewEventType),
                                                                       flutter::EncodableValue("loadStateChanged")},
                                                                      {flutter::EncodableValue(kWebviewEventValue), static_cast<int>(loadState)}});

                                            event_sink_->Success(event); });

    webview_client_->OnHistoryChanged([this](WebviewClientHistoryChanged historyChanged)
                                      {
                                          const auto event = flutter::EncodableValue(
                                              flutter::EncodableMap{
                                                  {flutter::EncodableValue(kWebviewEventType),
                                                   flutter::EncodableValue("onHistoryChanged")},
                                                  {flutter::EncodableValue(kWebviewEventValue),
                                                   flutter::EncodableValue(flutter::EncodableMap{
                                                       {flutter::EncodableValue("canGoBack"),
                                                        flutter::EncodableValue(
                                                            static_cast<bool>(historyChanged.can_go_back))},
                                                       {flutter::EncodableValue("canGoForward"),
                                                        flutter::EncodableValue(
                                                            static_cast<bool>(historyChanged.can_go_forward))},
                                                   })},});

                                          event_sink_->Success(event); });

    webview_client_->OnUrlChanged([this](const std::string &url)
                                  {
                                      const auto event = flutter::EncodableValue(
                                          flutter::EncodableMap{{flutter::EncodableValue(kWebviewEventType),
                                                                 flutter::EncodableValue("urlChanged")},
                                                                {flutter::EncodableValue(kWebviewEventValue), static_cast<std::string>(url)}});

                                      event_sink_->Success(event); });

    webview_client_->OnJSHandleChanged([this](const std::string &name)
                                       {
      // TODO 
        ::MessageBox(NULL, L"OnJSHandleChanged", L"Tip", MB_OK);
        // const auto event = flutter::EncodableValue(
        //     flutter::EncodableMap{
        //         {flutter::EncodableValue(kWebviewEventType),
        //          flutter::EncodableValue("javascriptChannelMessage")},
        //         {
        //             flutter::EncodableValue(kWebviewEventValue),
        //             flutter::EncodableValue(flutter::EncodableMap{
        //                 {flutter::EncodableValue("name"),
        //                  flutter::EncodableValue(
        //                      name)},
        //                 {flutter::EncodableValue("result"),
        //                  flutter::EncodableValue(
        //                      result)},
        //             }),
        //         },
        //     });
        // event_sink_->Success(event);
         });
}
